@@ -15,10 +15,14 @@
 # limitations under the License.
 #
 
-
+from google.cloud.osconfig_v1.services.os_config_service.async_client import (
+    OsConfigServiceAsyncClient,
+)
 from google.cloud.osconfig_v1.services.os_config_service.client import (
     OsConfigServiceClient,
 )
+from google.cloud.osconfig_v1.types.inventory import Inventory
+from google.cloud.osconfig_v1.types.osconfig_common import FixedOrPercent
 from google.cloud.osconfig_v1.types.patch_deployments import (
     CreatePatchDeploymentRequest,
 )
@@ -55,6 +59,7 @@ from google.cloud.osconfig_v1.types.patch_jobs import PatchConfig
 from google.cloud.osconfig_v1.types.patch_jobs import PatchInstanceFilter
 from google.cloud.osconfig_v1.types.patch_jobs import PatchJob
 from google.cloud.osconfig_v1.types.patch_jobs import PatchJobInstanceDetails
+from google.cloud.osconfig_v1.types.patch_jobs import PatchRollout
 from google.cloud.osconfig_v1.types.patch_jobs import WindowsUpdateSettings
 from google.cloud.osconfig_v1.types.patch_jobs import YumSettings
 from google.cloud.osconfig_v1.types.patch_jobs import ZypperSettings
@@ -67,11 +72,13 @@ __all__ = (
     "ExecStep",
     "ExecStepConfig",
     "ExecutePatchJobRequest",
+    "FixedOrPercent",
     "GcsObject",
     "GetPatchDeploymentRequest",
     "GetPatchJobRequest",
     "GooSettings",
     "Instance",
+    "Inventory",
     "ListPatchDeploymentsRequest",
     "ListPatchDeploymentsResponse",
     "ListPatchJobInstanceDetailsRequest",
@@ -80,12 +87,14 @@ __all__ = (
     "ListPatchJobsResponse",
     "MonthlySchedule",
     "OneTimeSchedule",
+    "OsConfigServiceAsyncClient",
     "OsConfigServiceClient",
     "PatchConfig",
     "PatchDeployment",
     "PatchInstanceFilter",
     "PatchJob",
     "PatchJobInstanceDetails",
+    "PatchRollout",
     "RecurringSchedule",
     "WeekDayOfMonth",
     "WeeklySchedule",
