@@ -16,5 +16,9 @@
 #
 
 from .client import OsConfigServiceClient
+from .async_client import OsConfigServiceAsyncClient
 
-__all__ = ("OsConfigServiceClient",)
+__all__ = (
+    "OsConfigServiceClient",
+    "OsConfigServiceAsyncClient",
+)
