@@ -20,11 +20,16 @@ from typing import Dict, Type
 
 from .base import OsConfigServiceTransport
 from .grpc import OsConfigServiceGrpcTransport
+from .grpc_asyncio import OsConfigServiceGrpcAsyncIOTransport
 
 
 # Compile a registry of transports.
 _transport_registry = OrderedDict()  # type: Dict[str, Type[OsConfigServiceTransport]]
 _transport_registry["grpc"] = OsConfigServiceGrpcTransport
+_transport_registry["grpc_asyncio"] = OsConfigServiceGrpcAsyncIOTransport
 
-
-__all__ = ("OsConfigServiceTransport", "OsConfigServiceGrpcTransport")
+__all__ = (
+    "OsConfigServiceTransport",
+    "OsConfigServiceGrpcTransport",
+    "OsConfigServiceGrpcAsyncIOTransport",
+)
