@@ -15,20 +15,22 @@
 # limitations under the License.
 #
 
-from typing import Callable, Dict, Tuple
+import warnings
+from typing import Callable, Dict, Optional, Sequence, Tuple
 
 from google.api_core import grpc_helpers  # type: ignore
+from google.api_core import gapic_v1  # type: ignore
+from google import auth  # type: ignore
 from google.auth import credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 
-
 import grpc  # type: ignore
 
 from google.cloud.osconfig_v1.types import patch_deployments
 from google.cloud.osconfig_v1.types import patch_jobs
 from google.protobuf import empty_pb2 as empty  # type: ignore
 
-from .base import OsConfigServiceTransport
+from .base import OsConfigServiceTransport, DEFAULT_CLIENT_INFO
 
 
 class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
@@ -47,14 +49,21 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
     top of HTTP/2); the ``grpcio`` package must be installed.
     """
 
+    _stubs: Dict[str, Callable]
+
     def __init__(
         self,
         *,
         host: str = "osconfig.googleapis.com",
         credentials: credentials.Credentials = None,
+        credentials_file: str = None,
+        scopes: Sequence[str] = None,
         channel: grpc.Channel = None,
         api_mtls_endpoint: str = None,
-        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None
+        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None,
+        ssl_channel_credentials: grpc.ChannelCredentials = None,
+        quota_project_id: Optional[str] = None,
+        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
     ) -> None:
         """Instantiate the transport.
 
@@ -66,21 +75,39 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.
                 This argument is ignored if ``channel`` is provided.
+            credentials_file (Optional[str]): A file with credentials that can
+                be loaded with :func:`google.auth.load_credentials_from_file`.
+                This argument is ignored if ``channel`` is provided.
+            scopes (Optional(Sequence[str])): A list of scopes. This argument is
+                ignored if ``channel`` is provided.
             channel (Optional[grpc.Channel]): A ``Channel`` instance through
                 which to make calls.
-            api_mtls_endpoint (Optional[str]): The mutual TLS endpoint. If
-                provided, it overrides the ``host`` argument and tries to create
+            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
+                If provided, it overrides the ``host`` argument and tries to create
                 a mutual TLS channel with client SSL credentials from
                 ``client_cert_source`` or applicatin default SSL credentials.
-            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]): A
-                callback to provide client SSL certificate bytes and private key
-                bytes, both in PEM format. It is ignored if ``api_mtls_endpoint``
-                is None.
+            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
+                Deprecated. A callback to provide client SSL certificate bytes and
+                private key bytes, both in PEM format. It is ignored if
+                ``api_mtls_endpoint`` is None.
+            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
+                for grpc channel. It is ignored if ``channel`` is provided.
+            quota_project_id (Optional[str]): An optional project to use for billing
+                and quota.
+            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
+                The client info used to send a user-agent string along with
+                API requests. If ``None``, then default info will be used.
+                Generally, you only need to set this if you're developing
+                your own client library.
 
         Raises:
-          google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
+          google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
               creation failed for any reason.
+          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
+              and ``credentials_file`` are passed.
         """
+        self._ssl_channel_credentials = ssl_channel_credentials
+
         if channel:
             # Sanity check: Ensure that channel and credentials are not both
             # provided.
@@ -88,13 +115,24 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
             # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
+            self._ssl_channel_credentials = None
         elif api_mtls_endpoint:
+            warnings.warn(
+                "api_mtls_endpoint and client_cert_source are deprecated",
+                DeprecationWarning,
+            )
+
             host = (
                 api_mtls_endpoint
                 if ":" in api_mtls_endpoint
                 else api_mtls_endpoint + ":443"
             )
 
+            if credentials is None:
+                credentials, _ = auth.default(
+                    scopes=self.AUTH_SCOPES, quota_project_id=quota_project_id
+                )
+
             # Create SSL credentials with client_cert_source or application
             # default SSL credentials.
             if client_cert_source:
@@ -106,61 +144,107 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
                 ssl_credentials = SslCredentials().ssl_credentials
 
             # create a new channel. The provided one is ignored.
-            self._grpc_channel = grpc_helpers.create_channel(
+            self._grpc_channel = type(self).create_channel(
                 host,
                 credentials=credentials,
+                credentials_file=credentials_file,
                 ssl_credentials=ssl_credentials,
-                scopes=self.AUTH_SCOPES,
+                scopes=scopes or self.AUTH_SCOPES,
+                quota_project_id=quota_project_id,
+                options=[
+                    ("grpc.max_send_message_length", -1),
+                    ("grpc.max_receive_message_length", -1),
+                ],
+            )
+            self._ssl_channel_credentials = ssl_credentials
+        else:
+            host = host if ":" in host else host + ":443"
+
+            if credentials is None:
+                credentials, _ = auth.default(
+                    scopes=self.AUTH_SCOPES, quota_project_id=quota_project_id
+                )
+
+            # create a new channel. The provided one is ignored.
+            self._grpc_channel = type(self).create_channel(
+                host,
+                credentials=credentials,
+                credentials_file=credentials_file,
+                ssl_credentials=ssl_channel_credentials,
+                scopes=scopes or self.AUTH_SCOPES,
+                quota_project_id=quota_project_id,
+                options=[
+                    ("grpc.max_send_message_length", -1),
+                    ("grpc.max_receive_message_length", -1),
+                ],
             )
 
-        # Run the base constructor.
-        super().__init__(host=host, credentials=credentials)
         self._stubs = {}  # type: Dict[str, Callable]
 
+        # Run the base constructor.
+        super().__init__(
+            host=host,
+            credentials=credentials,
+            credentials_file=credentials_file,
+            scopes=scopes or self.AUTH_SCOPES,
+            quota_project_id=quota_project_id,
+            client_info=client_info,
+        )
+
     @classmethod
     def create_channel(
         cls,
         host: str = "osconfig.googleapis.com",
         credentials: credentials.Credentials = None,
-        **kwargs
+        credentials_file: str = None,
+        scopes: Optional[Sequence[str]] = None,
+        quota_project_id: Optional[str] = None,
+        **kwargs,
     ) -> grpc.Channel:
         """Create and return a gRPC channel object.
         Args:
-            address (Optionsl[str]): The host for the channel to use.
+            address (Optional[str]): The host for the channel to use.
             credentials (Optional[~.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify this application to the service. If
                 none are specified, the client will attempt to ascertain
                 the credentials from the environment.
+            credentials_file (Optional[str]): A file with credentials that can
+                be loaded with :func:`google.auth.load_credentials_from_file`.
+                This argument is mutually exclusive with credentials.
+            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
+                service. These are only used when credentials are not specified and
+                are passed to :func:`google.auth.default`.
+            quota_project_id (Optional[str]): An optional project to use for billing
+                and quota.
             kwargs (Optional[dict]): Keyword arguments, which are passed to the
                 channel creation.
         Returns:
             grpc.Channel: A gRPC channel object.
+
+        Raises:
+            google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
+              and ``credentials_file`` are passed.
         """
+        scopes = scopes or cls.AUTH_SCOPES
         return grpc_helpers.create_channel(
-            host, credentials=credentials, scopes=cls.AUTH_SCOPES, **kwargs
+            host,
+            credentials=credentials,
+            credentials_file=credentials_file,
+            scopes=scopes,
+            quota_project_id=quota_project_id,
+            **kwargs,
         )
 
     @property
     def grpc_channel(self) -> grpc.Channel:
-        """Create the channel designed to connect to this service.
-
-        This property caches on the instance; repeated calls return
-        the same channel.
+        """Return the channel designed to connect to this service.
         """
-        # Sanity check: Only create a new channel if we do not already
-        # have one.
-        if not hasattr(self, "_grpc_channel"):
-            self._grpc_channel = self.create_channel(
-                self._host, credentials=self._credentials
-            )
-
-        # Return the channel from cache.
         return self._grpc_channel
 
     @property
     def execute_patch_job(
-        self
+        self,
     ) -> Callable[[patch_jobs.ExecutePatchJobRequest], patch_jobs.PatchJob]:
         r"""Return a callable for the execute patch job method over gRPC.
 
@@ -187,7 +271,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def get_patch_job(
-        self
+        self,
     ) -> Callable[[patch_jobs.GetPatchJobRequest], patch_jobs.PatchJob]:
         r"""Return a callable for the get patch job method over gRPC.
 
@@ -215,7 +299,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def cancel_patch_job(
-        self
+        self,
     ) -> Callable[[patch_jobs.CancelPatchJobRequest], patch_jobs.PatchJob]:
         r"""Return a callable for the cancel patch job method over gRPC.
 
@@ -242,7 +326,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def list_patch_jobs(
-        self
+        self,
     ) -> Callable[[patch_jobs.ListPatchJobsRequest], patch_jobs.ListPatchJobsResponse]:
         r"""Return a callable for the list patch jobs method over gRPC.
 
@@ -268,7 +352,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def list_patch_job_instance_details(
-        self
+        self,
     ) -> Callable[
         [patch_jobs.ListPatchJobInstanceDetailsRequest],
         patch_jobs.ListPatchJobInstanceDetailsResponse,
@@ -300,7 +384,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def create_patch_deployment(
-        self
+        self,
     ) -> Callable[
         [patch_deployments.CreatePatchDeploymentRequest],
         patch_deployments.PatchDeployment,
@@ -329,7 +413,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def get_patch_deployment(
-        self
+        self,
     ) -> Callable[
         [patch_deployments.GetPatchDeploymentRequest], patch_deployments.PatchDeployment
     ]:
@@ -357,7 +441,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def list_patch_deployments(
-        self
+        self,
     ) -> Callable[
         [patch_deployments.ListPatchDeploymentsRequest],
         patch_deployments.ListPatchDeploymentsResponse,
@@ -386,7 +470,7 @@ class OsConfigServiceGrpcTransport(OsConfigServiceTransport):
 
     @property
     def delete_patch_deployment(
-        self
+        self,
     ) -> Callable[[patch_deployments.DeletePatchDeploymentRequest], empty.Empty]:
         r"""Return a callable for the delete patch deployment method over gRPC.
 
